(function (global, factory) {
typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('ramda'), require('ramda-extension'), require('@redux-tools/namespaces'), require('@redux-tools/utils'), require('invariant')) :
typeof define === 'function' && define.amd ? define(['exports', 'ramda', 'ramda-extension', '@redux-tools/namespaces', '@redux-tools/utils', 'invariant'], factory) :
(global = global || self, factory(global.ReduxToolsInjectors = {}, global.R, global.R_, global.ReduxToolsNamespaces, global.ReduxToolsUtils, global.invariant));
}(this, function (exports, ramda, ramdaExtension, namespaces, utils, invariant) { 'use strict';

invariant = invariant && invariant.hasOwnProperty('default') ? invariant['default'] : invariant;

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var FUNCTION_KEY = '@@redux-tools/injectors/FUNCTION_KEY';

var createEntry = ramda.applySpec({
  key: ramda.nth(0),
  value: ramda.nth(1)
});
/**
 * Converts the input of `store.injectSomething()` or `store.ejectSomething()`
 * to an array of standalone entries.
 *
 * @param {Object|Function} injectables an object or function with injectables as values
 * @param {Object} props props to store in the entry, e.g. `namespace` or `feature`
 * @returns {Object[]} an array of entries
 */

var createEntries = function createEntries(injectables, _ref) {
  if (_ref === void 0) {
    _ref = {};
  }

  var _ref2 = _ref,
      feature = _ref2.feature,
      otherProps = _objectWithoutPropertiesLoose(_ref2, ["feature"]);

  var sanitizedProps = ramdaExtension.rejectNil(_extends({}, otherProps, {
    feature: feature || namespaces.DEFAULT_FEATURE
  }));
  var createEntriesFromObject = ramda.compose(ramda.map(ramda.merge(sanitizedProps)), ramda.map(createEntry), ramda.toPairs);

  var createEntriesFromFunction = function createEntriesFromFunction(injectable) {
    return [_extends({
      key: FUNCTION_KEY,
      value: injectable
    }, sanitizedProps)];
  };

  return ramda.ifElse(ramdaExtension.isFunction, createEntriesFromFunction, createEntriesFromObject)(injectables);
};

var enhanceStore = function enhanceStore(prevStore, storeInterface, _temp) {
  var _extends2;

  var _ref = _temp === void 0 ? {} : _temp,
      _ref$onEjected = _ref.onEjected,
      onEjected = _ref$onEjected === void 0 ? ramdaExtension.noop : _ref$onEjected,
      _ref$onInjected = _ref.onInjected,
      onInjected = _ref$onInjected === void 0 ? ramdaExtension.noop : _ref$onInjected;

  invariant(ramdaExtension.isObject(prevStore), 'You must pass a Redux store as the first argument to `enhanceStore()`');
  invariant(ramdaExtension.isObject(storeInterface), 'You must pass a store interface as the second argument to `enhanceStore()`');
  var injectionKey = storeInterface.injectionKey,
      ejectionKey = storeInterface.ejectionKey,
      getEntries = storeInterface.getEntries,
      setEntries = storeInterface.setEntries,
      type = storeInterface.type;
  var _prevStore$dispatch = prevStore.dispatch,
      dispatch = _prevStore$dispatch === void 0 ? ramdaExtension.noop : _prevStore$dispatch;
  var actionType = ramdaExtension.toScreamingSnakeCase(type);

  var inject = function inject(injectables, props) {
    if (props === void 0) {
      props = {};
    }

    var entries = createEntries(injectables, props);
    ramda.forEach(function (entry) {
      return invariant( // NOTE: `isFunction` from ramda-extension returns `false` for `jest.fn()`.
      typeof entry.value === 'function', "Injecting " + type + ", but the value of " + JSON.stringify(entry) + " is not a function.");
    }, entries);
    var nextEntries = ramda.concat(getEntries(nextStore), entries);
    setEntries(nextEntries, nextStore);
    onInjected({
      injectables: injectables,
      props: props,
      entries: entries
    });
    dispatch({
      type: "@redux-tools/" + actionType + "_INJECTED",
      payload: ramda.keys(injectables),
      meta: props
    });
  };

  var eject = function eject(injectables, props) {
    if (props === void 0) {
      props = {};
    }

    var entries = createEntries(injectables, props);
    var nextEntries = utils.withoutOnce(entries, getEntries(nextStore));
    setEntries(nextEntries, nextStore);
    onEjected({
      injectables: injectables,
      props: props,
      entries: entries
    });
    dispatch({
      type: "@redux-tools/" + actionType + "_EJECTED",
      payload: typeof injectables === 'function' ? null : ramda.keys(injectables),
      meta: props
    });
  };

  var nextStore = _extends({}, prevStore, (_extends2 = {}, _extends2[injectionKey] = inject, _extends2[ejectionKey] = eject, _extends2));

  setEntries([], nextStore);
  return nextStore;
};

var makeStoreInterface = function makeStoreInterface(type) {
  invariant(type, 'The type of the injectables must be defined.');
  return {
    type: type,
    injectionKey: "inject" + ramdaExtension.toPascalCase(type),
    ejectionKey: "eject" + ramdaExtension.toPascalCase(type),
    getEntries: ramda.o(ramdaExtension.defaultToEmptyArray, ramda.path(['entries', type])),
    setEntries: ramda.curry(function (entries, store) {
      var _extends2;

      return store.entries = _extends({}, store.entries, (_extends2 = {}, _extends2[type] = entries, _extends2));
    })
  };
};

exports.FUNCTION_KEY = FUNCTION_KEY;
exports.createEntries = createEntries;
exports.enhanceStore = enhanceStore;
exports.makeStoreInterface = makeStoreInterface;

Object.defineProperty(exports, '__esModule', { value: true });

}));
